@@ -91,7 +91,7 @@ Workbook$methods(setColWidths = function(sheet) {
         sd <- worksheets[[sheet]]$sheet_data
 
         if (length(merge_cols) > 0) {
-          all_merged_cells <- lapply(1:length(merge_cols), function(i) {
+          all_merged_cells <- lapply(seq_along(merge_cols), function(i) {
             expand.grid(
               "rows" = min(merge_rows[[i]]):max(merge_rows[[i]]),
               "cols" = min(merge_cols[[i]]):max(merge_cols[[i]])
@@ -125,8 +125,8 @@ Workbook$methods(setColWidths = function(sheet) {
         autoColumns = as.integer(autoCols),
         widths = allCharWidths,
         baseFontCharWidth = baseFontCharWidth,
-        minW = getOption("openxlsx.minWidth", 3),
-        maxW = getOption("openxlsx.maxWidth", 250)
+        minW = openxlsx_getOp("minWidth", 3),
+        maxW = openxlsx_getOp("maxWidth", 250)
       )
 
       missingAuto <- autoCols[!autoCols %in% names(calculatedWidths)]
