@@ -10,19 +10,20 @@
 #' @param startRow A vector specifying the starting row to write df
 #' @param xy An alternative to specifying startCol and startRow individually.
 #' A vector of the form c(startCol, startRow)
-#' @param colNames If \code{TRUE}, column names of x are written.
-#' @param rowNames If \code{TRUE}, row names of x are written.
+#' @param colNames If `TRUE`, column names of x are written.
+#' @param rowNames If `TRUE`, row names of x are written.
+#' @param row.names,col.names Deprecated, please use `rowNames`, `colNames` instead
 #' @param tableStyle Any excel table style name or "none" (see "formatting" vignette).
 #' @param tableName name of table in workbook. The table name must be unique.
 #' @param headerStyle Custom style to apply to column names.
-#' @param withFilter If \code{TRUE}, columns with have filters in the first row.
-#' @param keepNA If \code{TRUE}, NA values are converted to #N/A (or \code{na.string}, if not NULL) in Excel, else NA cells will be empty.
-#' @param na.string If not NULL, and if \code{keepNA} is \code{TRUE}, NA values are converted to this string in Excel.
+#' @param withFilter If `TRUE` or `NA`, columns with have filters in the first row.
+#' @param keepNA If `TRUE`, NA values are converted to #N/A (or `na.string`, if not NULL) in Excel, else NA cells will be empty.
+#' @param na.string If not NULL, and if `keepNA` is `TRUE`, NA values are converted to this string in Excel.
 #' @param sep Only applies to list columns. The separator used to collapse list columns to a character vector e.g. sapply(x$list_column, paste, collapse = sep).
-#' @param stack If \code{TRUE} the new style is merged with any existing cell styles.  If FALSE, any
+#' @param stack If `TRUE` the new style is merged with any existing cell styles.  If FALSE, any
 #' existing style is replaced by the new style.
 #' \cr\cr
-#' \cr\bold{The below options correspond to Excel table options:}
+#' \cr**The below options correspond to Excel table options:**
 #' \cr
 #' \if{html}{\figure{tableoptions.png}{options: width="40\%" alt="Figure: table_options.png"}}
 #' \if{latex}{\figure{tableoptions.pdf}{options: width=7cm}}
@@ -34,11 +35,10 @@
 #' @details columns of x with class Date/POSIXt, currency, accounting,
 #' hyperlink, percentage are automatically styled as dates, currency, accounting,
 #' hyperlinks, percentages respectively.
-#' @seealso \code{\link{addWorksheet}}
-#' @seealso \code{\link{writeData}}
-#' @seealso \code{\link{removeTable}}
-#' @seealso \code{\link{getTables}}
-#' @importFrom stats na.omit
+#' @seealso [addWorksheet()]
+#' @seealso [writeData()]
+#' @seealso [removeTable()]
+#' @seealso [getTables()]
 #' @export
 #' @examples
 #' ## see package vignettes for further examples.
@@ -135,24 +135,54 @@
 #' saveWorkbook(wb, file = "tableStylesGallery.xlsx", overwrite = TRUE)
 #' }
 #'
-writeDataTable <- function(wb, sheet, x,
-                           startCol = 1,
-                           startRow = 1,
-                           xy = NULL,
-                           colNames = TRUE,
-                           rowNames = FALSE,
-                           tableStyle = "TableStyleLight9",
-                           tableName = NULL,
-                           headerStyle = NULL,
-                           withFilter = TRUE,
-                           keepNA = FALSE,
-                           na.string = NULL,
-                           sep = ", ",
-                           stack = FALSE,
-                           firstColumn = FALSE,
-                           lastColumn = FALSE,
-                           bandedRows = TRUE,
-                           bandedCols = FALSE) {
+writeDataTable <- function(
+  wb,
+  sheet,
+  x,
+  startCol    = 1,
+  startRow    = 1,
+  xy          = NULL,
+  colNames    = TRUE,
+  rowNames    = FALSE,
+  tableStyle  = openxlsx_getOp("tableStyle", "TableStyleLight9"),
+  tableName   = NULL,
+  headerStyle = openxlsx_getOp("headerStyle"),
+  withFilter  = openxlsx_getOp("withFilter", TRUE),
+  keepNA      = openxlsx_getOp("keepNA", FALSE),
+  na.string   = openxlsx_getOp("na.string"),
+  sep         = ", ",
+  stack       = FALSE,
+  firstColumn = openxlsx_getOp("firstColumn", FALSE),
+  lastColumn  = openxlsx_getOp("lastColumn", FALSE),
+  bandedRows  = openxlsx_getOp("bandedRows", TRUE),
+  bandedCols  = openxlsx_getOp("bandedCols", FALSE),
+  col.names,
+  row.names
+  ) {
+  op <- get_set_options()
+  on.exit(options(op), add = TRUE)
+  
+  ## increase scipen to avoid writing in scientific
+  
+  if (!missing(row.names)) {
+    warning("Please use 'rowNames' instead of 'row.names'", call. = FALSE)
+    row.names <- rowNames
+  }
+  
+  if (!missing(col.names)) {
+    warning("Please use 'colNames' instead of 'col.names'", call. = FALSE)
+    colNames <- col.names
+  }
+  
+  # Set NULLs
+  withFilter  <- withFilter  %||% TRUE
+  keepNA      <- keepNA      %||% FALSE
+  firstColumn <- firstColumn %||% FALSE
+  lastColumn  <- lastColumn  %||% FALSE
+  bandedRows  <- bandedRows  %||% TRUE
+  bandedCols  <- bandedCols  %||% FALSE
+  withFilter  <- withFilter  %||% TRUE
+  
   if (!is.null(xy)) {
     if (length(xy) != 2) {
       stop("xy parameter must have length 2")
@@ -161,40 +191,25 @@ writeDataTable <- function(wb, sheet, x,
     startRow <- xy[[2]]
   }
 
-  ## Input validating
-  if (!"Workbook" %in% class(wb)) stop("First argument must be a Workbook.")
-  if (!"data.frame" %in% class(x)) stop("x must be a data.frame.")
-  if (!is.logical(colNames)) stop("colNames must be a logical.")
-  if (!is.logical(rowNames)) stop("rowNames must be a logical.")
-  if (!is.null(headerStyle) & !"Style" %in% class(headerStyle)) stop("headerStyle must be a style object or NULL.")
-  if (!is.logical(withFilter)) stop("withFilter must be a logical.")
-  if ((!is.character(sep)) | (length(sep) != 1)) stop("sep must be a character vector of length 1")
-
-  if (!is.logical(firstColumn)) stop("firstColumn must be a logical.")
-  if (!is.logical(lastColumn)) stop("lastColumn must be a logical.")
-  if (!is.logical(bandedRows)) stop("bandedRows must be a logical.")
-  if (!is.logical(bandedCols)) stop("bandedCols must be a logical.")
-
+  # Assert parameters
+  assert_class(wb, "Workbook")
+  assert_class(x, "data.frame")
+  assert_true_false(colNames)
+  assert_true_false(rowNames)
+  assert_class(headerStyle, "Style", or_null = TRUE)
+  assert_true_false(withFilter)
+  assert_character1(sep)
+  assert_true_false(firstColumn)
+  assert_true_false(lastColumn)
+  assert_true_false(bandedRows)
+  assert_true_false(bandedCols)
+  
   if (is.null(tableName)) {
-    tableName <- paste0("Table", as.character(length(wb$tables) + 3L))
+    tableName <- sprintf("Table%i", length(wb$tables) + 3L)
   } else {
     tableName <- wb$validate_table_name(tableName)
   }
 
-
-  ## increase scipen to avoid writing in scientific
-  exSciPen <- getOption("scipen")
-  od <- getOption("OutDec")
-  exDigits <- getOption("digits")
-
-  options("scipen" = 200)
-  options("OutDec" = ".")
-  options("digits" = 22)
-
-  on.exit(options("scipen" = exSciPen), add = TRUE)
-  on.exit(expr = options("OutDec" = od), add = TRUE)
-  on.exit(options("digits" = exDigits), add = TRUE)
-
   ## convert startRow and startCol
   if (!is.numeric(startCol)) {
     startCol <- convertFromExcelRef(startCol)
@@ -207,25 +222,17 @@ writeDataTable <- function(wb, sheet, x,
   }
 
   ## If 0 rows append a blank row
-
-  validNames <- c("none", paste0("TableStyleLight", 1:21), paste0("TableStyleMedium", 1:28), paste0("TableStyleDark", 1:11))
-  if (!tolower(tableStyle) %in% tolower(validNames)) {
-    stop("Invalid table style.")
-  } else {
-    tableStyle <- validNames[grepl(paste0("^", tableStyle, "$"), validNames, ignore.case = TRUE)]
-  }
-
-  tableStyle <- na.omit(tableStyle)
-  if (length(tableStyle) == 0) {
-    stop("Unknown table style.")
-  }
+  
+  tableStyle <- validate_StyleName(tableStyle)
 
   ## header style
-  if ("Style" %in% class(headerStyle)) {
+  if (inherits(headerStyle, "Style")) {
     addStyle(
-      wb = wb, sheet = sheet, style = headerStyle,
-      rows = startRow,
-      cols = 0:(ncol(x) - 1L) + startCol,
+      wb         = wb, 
+      sheet      = sheet,
+      style      = headerStyle,
+      rows       = startRow,
+      cols       = 0:(ncol(x) - 1L) + startCol,
       gridExpand = TRUE
     )
   }
@@ -234,9 +241,7 @@ writeDataTable <- function(wb, sheet, x,
 
   if (colNames) {
     colNames <- colnames(x)
-    if (any(duplicated(tolower(colNames)))) {
-      stop("Column names of x must be case-insensitive unique.")
-    }
+    assert_unique(colNames, case_sensitive = FALSE)
 
     ## zero char names are invalid
     char0 <- nchar(colNames) == 0
@@ -244,12 +249,16 @@ writeDataTable <- function(wb, sheet, x,
       colNames[char0] <- colnames(x)[char0] <- paste0("Column", which(char0))
     }
   } else {
-    colNames <- paste0("Column", 1:ncol(x))
+    colNames <- paste0("Column", seq_along(x))
     names(x) <- colNames
   }
+  
   ## If zero rows, append an empty row (prevent XML from corrupting)
   if (nrow(x) == 0) {
-    x <- rbind(as.data.frame(x), matrix("", nrow = 1, ncol = ncol(x), dimnames = list(character(), colnames(x))))
+    x <- rbind(
+      as.data.frame(x),
+      matrix("", nrow = 1, ncol = ncol(x), dimnames = list(character(), colnames(x)))
+    )
     names(x) <- colNames
   }
 
@@ -260,27 +269,37 @@ writeDataTable <- function(wb, sheet, x,
   ## check not overwriting another table
   wb$check_overwrite_tables(
     sheet = sheet,
-    new_rows = c(startRow, startRow + nrow(x) - 1L + 1L) ## + header
-    , new_cols = c(startCol, startCol + ncol(x) - 1L)
+    new_rows = c(startRow, startRow + nrow(x) - 1L + 1L), ## + header
+    new_cols = c(startCol, startCol + ncol(x) - 1L)
   )
 
 
   ## column class styling
+  # consider not using lowercase and instead use inherits(x, class)
   colClasses <- lapply(x, function(x) tolower(class(x)))
-  classStyles(wb, sheet = sheet, startRow = startRow, startCol = startCol, colNames = TRUE, nRow = nrow(x), colClasses = colClasses, stack = stack)
+  classStyles(
+    wb,
+    sheet      = sheet,
+    startRow   = startRow, 
+    startCol   = startCol, 
+    colNames   = TRUE,
+    nRow       = nrow(x),
+    colClasses = colClasses, 
+    stack      = stack
+  )
 
   ## write data to worksheet
   wb$writeData(
-    df = x,
-    colNames = TRUE,
-    sheet = sheet,
-    startRow = startRow,
-    startCol = startCol,
+    df         = x,
+    colNames   = TRUE,
+    sheet      = sheet,
+    startRow   = startRow,
+    startCol   = startCol,
     colClasses = colClasses,
     hlinkNames = NULL,
-    keepNA = keepNA,
-    na.string = na.string,
-    list_sep = sep
+    keepNA     = keepNA,
+    na.string  = na.string,
+    list_sep   = sep
   )
 
   ## replace invalid XML characters
@@ -288,17 +307,17 @@ writeDataTable <- function(wb, sheet, x,
 
   ## create table.xml and assign an id to worksheet tables
   wb$buildTable(
-    sheet = sheet,
-    colNames = colNames,
-    ref = ref,
-    showColNames = showColNames,
-    tableStyle = tableStyle,
-    tableName = tableName,
-    withFilter = withFilter[1],
-    totalsRowCount = 0L,
-    showFirstColumn = firstColumn[1],
-    showLastColumn = lastColumn[1],
-    showRowStripes = bandedRows[1],
+    sheet             = sheet,
+    colNames          = colNames,
+    ref               = ref,
+    showColNames      = showColNames,
+    tableStyle        = tableStyle,
+    tableName         = tableName,
+    withFilter        = withFilter[1],
+    totalsRowCount    = 0L,
+    showFirstColumn   = firstColumn[1],
+    showLastColumn    = lastColumn[1],
+    showRowStripes    = bandedRows[1],
     showColumnStripes = bandedCols[1]
   )
 }
